#!/bin/sh

# Component: drwatson
#
# Description:
#  System state monitoring
#
# Usage: monitor.sh [dumpOnly | cron_job] # dumpOnly is default
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$

. $HOME/etc/setenv.sh

usage="Usage: monitor.sh [dumpOnly | cron_job] # dumpOnly is default"
type="dumpOnly"

if [ -n "$1" ] ; then
	if [ "$1" = "-h" -o "$1" = "--help" ] ; then
		echo ; echo "$usage" ; echo
		exit 0
	fi
	type="$1"
fi

# do not run if another monitor.sh runs
if [ -f "${LOCK_VAR}/monitor.sh.lock" ]; then
    lockFileContent=`cat ${LOCK_VAR}/monitor.sh.lock`
    #echo "lock file exists, content (pid): ${lockFileContent}"
    if [ -z "${lockFileContent}" ]; then
	#echo "content is empty, we can delete the lock file"
	rm -f ${LOCK_VAR}/monitor.sh.lock
    else
	# try to check if process with that id exists and is a monitor.sh
	existingProcessName=`ps -p "${lockFileContent}" -o comm=`
	#echo "process name read from ps command by monitor lock file content pid: ${existingProcessName}"
	# check if another monitor.sh is still running with lock file content pid
	if [ "${existingProcessName}" = "$(basename "$0")" ]; then
	    #echo "running process name is the same as this process name, previous monitoring is still running, exit with -1 for now"
	    exit 2
	else
	    #echo "no other running monitor process was found for lock file content pid, we can delete the lock file"
	    rm -f ${LOCK_VAR}/monitor.sh.lock
	fi
    fi
fi

$UTILS_HOME/bin/createLockFile "${LOCK_VAR}/monitor.sh.lock"
[ $? -ne 0 ] && exit 1 # the flag file already exists or could not be created
# give owner permission to write pid into it
chmod u+w "${LOCK_VAR}/monitor.sh.lock"
myName=$(basename "$0")
myProcessId=`ps -C "$myName" -o pid=`
printf $myProcessId >> "${LOCK_VAR}/monitor.sh.lock"
chmod u-w "${LOCK_VAR}/monitor.sh.lock"


if [ "$type" = "dumpOnly" ] ; then
	$ETC/rc monitor
	rm -f ${LOCK_VAR}/monitor.sh.lock
	exit 0
fi


# check monitoring_rate and monitoring_counter at cron job

# monitoring rate configuration - default=1 means : always collect and log monitor info
monitoringRate=1
monitoringRateCfg="$CONF/monitoring_rate.cfg"
[ -f "$monitoringRateCfg" ] && monitoringRate=`cat $monitoringRateCfg`

if [ $monitoringRate -eq 0 ] ; then
	rm -f ${LOCK_VAR}/monitor.sh.lock
	exit 0
fi	 

monitoringCounter=0
monitoringCounterFile="$TMP/monitoring_counter"
[ -f "$monitoringCounterFile" ] && monitoringCounter=`cat $monitoringCounterFile`

monitoringCounter=$(( $monitoringCounter + 1 ))

if [ $monitoringCounter -lt $monitoringRate ] ; then
  echo "$monitoringCounter" >$monitoringCounterFile
  rm -f ${LOCK_VAR}/monitor.sh.lock
  exit 0
else	# Will Run!
  echo "0" >$monitoringCounterFile
  sleep 5	# Don't start along with another cron job!
fi

tempLogFile="$TMP/monitor.$$.log.tmp"

startDateTimeLogLine=`date +"monitorSTART=%Y/%m/%d %T"`

echo >$tempLogFile
echo "$startDateTimeLogLine" >>$tempLogFile
echo >>$tempLogFile
$ETC/rc monitor >>$tempLogFile

cat $tempLogFile >>$LOG/monitor.log


### --- Filter by Critical Limits

criticalLimitFile=""
[ -f "$CONF/crucialStage.cfg" ] && criticalLimitFile="$CONF/crucialStage.cfg"
[ -f "$DRWATSON_CONFIG/crucialStage.cfg" ] && criticalLimitFile="$DRWATSON_CONFIG/crucialStage.cfg"
if [ -z "$criticalLimitFile" ] ; then 
	rm -f ${LOCK_VAR}/monitor.sh.lock
	exit 0
fi	

criticalValueFile="$TMP/capture.$$.res"

cat $criticalLimitFile $tempLogFile | grep "=" | LC_NUMERIC=en_US awk -vFS="=" '
{
	if ($1 in limitsArray)
	{
		if ( limitsArray[$1] < $2 )
		{
			print $0;
		}
	}
	else
		limitsArray[$1]=$2;
}' >$criticalValueFile
															
criticalValue=`cat $criticalValueFile`
if [ -z "$criticalValue" ] ; then
	rm -f $criticalValueFile
	rm -f $tempLogFile
	cat $criticalLimitFile | grep "=" | awk -vFS="=" '{print $1}' |\
	while read drWatsonFlagFile ; do
		rm -f $TMP/$drWatsonFlagFile
	done
	rm -f ${LOCK_VAR}/monitor.sh.lock
	exit 0
else
	echo "$startDateTimeLogLine" >>$criticalValueFile
fi
																							
$DRWATSON_HOME/capture.sh emergency $criticalValueFile >>$LOG/capture.log 2>&1

rm -f $criticalValueFile
rm -f $tempLogFile
rm -f ${LOCK_VAR}/monitor.sh.lock
